@@ -112,6 +112,85 @@ FastMKS<KernelType, MatType, TreeType>::FastMKS(Tree* referenceTree,
   // Nothing to do.
 }
 
+template<typename KernelType,
+         typename MatType,
+         template<typename TreeMetricType,
+                  typename TreeStatType,
+                  typename TreeMatType> class TreeType>
+FastMKS<KernelType, MatType, TreeType>::FastMKS(const FastMKS& other) :
+    referenceSet(NULL),
+    referenceTree(other.referenceTree ? new Tree(*other.referenceTree) : NULL),
+    treeOwner(other.referenceTree != NULL),
+    setOwner(other.referenceTree == NULL),
+    singleMode(other.singleMode),
+    naive(other.naive),
+    metric(other.metric)
+{
+  // Set reference set correctly.
+  if (referenceTree)
+    referenceSet = &referenceTree->Dataset();
+  else
+    referenceSet = new MatType(*other.referenceSet);
+}
+
+template<typename KernelType,
+         typename MatType,
+         template<typename TreeMetricType,
+                  typename TreeStatType,
+                  typename TreeMatType> class TreeType>
+FastMKS<KernelType, MatType, TreeType>::FastMKS(FastMKS&& other) :
+    referenceSet(other.referenceSet),
+    referenceTree(other.referenceTree),
+    treeOwner(other.treeOwner),
+    setOwner(other.setOwner),
+    singleMode(other.singleMode),
+    naive(other.naive),
+    metric(std::move(other.metric))
+{
+  // Clear information from the other.
+  other.referenceSet = NULL;
+  other.referenceTree = NULL;
+  other.treeOwner = false;
+  other.setOwner = false;
+  other.singleMode = false;
+  other.naive = false;
+}
+
+template<typename KernelType,
+         typename MatType,
+         template<typename TreeMetricType,
+                  typename TreeStatType,
+                  typename TreeMatType> class TreeType>
+FastMKS<KernelType, MatType, TreeType>&
+FastMKS<KernelType, MatType, TreeType>::operator=(const FastMKS& other)
+{
+  // Clear anything we currently have.
+  if (treeOwner)
+    delete referenceTree;
+  if (setOwner)
+    delete referenceSet;
+
+  referenceTree = NULL;
+  referenceSet = NULL;
+
+  if (other.referenceTree)
+  {
+    referenceTree = new Tree(*other.referenceTree);
+    referenceSet = &referenceTree->Dataset();
+    treeOwner = true;
+    setOwner = false;
+  }
+  else
+  {
+    referenceSet = new MatType(*other.referenceSet);
+    treeOwner = false;
+    setOwner = true;
+  }
+
+  singleMode = other.singleMode;
+  naive = other.naive;
+}
+
 template<typename KernelType,
          typename MatType,
          template<typename TreeMetricType,
@@ -430,15 +509,13 @@ template<typename KernelType,
                   typename TreeStatType,
                   typename TreeMatType> class TreeType>
 template<typename Archive>
-void FastMKS<KernelType, MatType, TreeType>::Serialize(
+void FastMKS<KernelType, MatType, TreeType>::serialize(
     Archive& ar,
     const unsigned int /* version */)
 {
-  using data::CreateNVP;
-
   // Serialize preferences for search.
-  ar & CreateNVP(naive, "naive");
-  ar & CreateNVP(singleMode, "singleMode");
+  ar & BOOST_SERIALIZATION_NVP(naive);
+  ar & BOOST_SERIALIZATION_NVP(singleMode);
 
   // If we are doing naive search, serialize the dataset.  Otherwise we
   // serialize the tree.
@@ -452,8 +529,8 @@ void FastMKS<KernelType, MatType, TreeType>::Serialize(
       setOwner = true;
     }
 
-    ar & CreateNVP(referenceSet, "referenceSet");
-    ar & CreateNVP(metric, "metric");
+    ar & BOOST_SERIALIZATION_NVP(referenceSet);
+    ar & BOOST_SERIALIZATION_NVP(metric);
   }
   else
   {
@@ -466,7 +543,7 @@ void FastMKS<KernelType, MatType, TreeType>::Serialize(
       treeOwner = true;
     }
 
-    ar & CreateNVP(referenceTree, "referenceTree");
+    ar & BOOST_SERIALIZATION_NVP(referenceTree);
 
     if (Archive::is_loading::value)
     {
